/* eslint max-len: ["off"] no-unused-vars: ["off"] padded-blocks: ["off"] */
/*
 * Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not 
 * use this file except in compliance with the License. A copy of the License 
 * is located at
 *
 *    http://aws.amazon.com/apache2.0/
 *
 * or in the "license" file accompanying this file. This file is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either 
 * express or implied. See the License for the specific language governing 
 * permissions and limitations under the License.
 */
'use strict';

const Constants = require('../src/Constants');
const Encoders = require('../src/Encoders');





exports.write_authorizeConnection_1489122155_1 = function(accessKeyId, signature, stringToSign, sessionToken, userAgent, tube) {
  let encoder = new Encoders(tube.cbor);

  tube.write(tube.cbor.encodeInt(1));
  tube.write(tube.cbor.encodeInt(1489122155));
  tube.write(tube.cbor.encodeString(accessKeyId));

  tube.write(tube.cbor.encodeString(signature));

  tube.write(tube.cbor.encodeBinary(stringToSign));

  tube.write(sessionToken === null ? tube.cbor.encodeNull() : tube.cbor.encodeString(sessionToken));

  tube.write(userAgent === null ? tube.cbor.encodeNull() : tube.cbor.encodeString(userAgent));

  tube.flush();
};

exports.write_defineAttributeList_670678385_1 = function(attributeListId, tube) {
  let encoder = new Encoders(tube.cbor);

  tube.write(tube.cbor.encodeInt(1));
  tube.write(tube.cbor.encodeInt(670678385));
  tube.write(tube.cbor.encodeInt(attributeListId));

  tube.flush();
};

exports.write_defineAttributeListId_N1230579644_1 = function(attributeNames, tube) {
  let encoder = new Encoders(tube.cbor);

  tube.write(tube.cbor.encodeInt(1));
  tube.write(tube.cbor.encodeInt(-1230579644));
  tube.write(tube.cbor.encodeArray(attributeNames));

  tube.flush();
};

exports.write_defineKeySchema_N742646399_1 = function(tableName, tube) {
  let encoder = new Encoders(tube.cbor);

  tube.write(tube.cbor.encodeInt(1));
  tube.write(tube.cbor.encodeInt(-742646399));
  tube.write(tube.cbor.encodeBinary(tableName));

  tube.flush();
};

exports.write_deleteItem_1013539361_1 = function(request, tube) {
  let encoder = new Encoders(tube.cbor, request._keySchema);

  tube.write(tube.cbor.encodeInt(1));
  tube.write(tube.cbor.encodeInt(1013539361));
  tube.write(encoder.encodeTableName(request.TableName));

  tube.write(encoder.encodeKey(request.Key));

  let hasKwargs = request.ReturnValues || request.ReturnConsumedCapacity || request.ReturnItemCollectionMetrics || request.ConditionExpression || request.ExpressionAttributeNames || request.ExpressionAttributeValues;
  if(hasKwargs) {
    tube.write(tube.cbor.encodeMapStreamHeader());
    if(request.ReturnValues !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnValues));
  tube.write(tube.cbor.encodeInt(Constants.ReturnValuesValues[request.ReturnValues.toUpperCase()]));
}

    if(request.ReturnConsumedCapacity !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnConsumedCapacity));
  tube.write(tube.cbor.encodeInt(Constants.ReturnConsumedCapacityValues[request.ReturnConsumedCapacity.toUpperCase()]));
}

    if(request.ReturnItemCollectionMetrics !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnItemCollectionMetrics));
  tube.write(tube.cbor.encodeInt(Constants.ReturnItemCollectionMetricsValues[request.ReturnItemCollectionMetrics.toUpperCase()]));
}

    if(request.ConditionExpression !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ConditionExpression));
  tube.write(tube.cbor.encodeBinary(request.ConditionExpression));
}

    // This operation has expressions, so deal with those together
    let exprBytes = encoder.encodeExpressions(request).otherBytes;
    if(exprBytes !== null) {
      tube.write(exprBytes);
    }

    tube.write(tube.cbor.encodeStreamBreak());
  } else {
    tube.write(tube.cbor.encodeNull());
  }

  tube.flush();
};

exports.write_endpoints_455855874_1 = function(tube) {
  let encoder = new Encoders(tube.cbor);

  tube.write(tube.cbor.encodeInt(1));
  tube.write(tube.cbor.encodeInt(455855874));
  tube.flush();
};

exports.write_getItem_263244906_1 = function(request, tube) {
  let encoder = new Encoders(tube.cbor, request._keySchema);

  tube.write(tube.cbor.encodeInt(1));
  tube.write(tube.cbor.encodeInt(263244906));
  tube.write(encoder.encodeTableName(request.TableName));

  tube.write(encoder.encodeKey(request.Key));

  let hasKwargs = request.ConsistentRead || request.ReturnConsumedCapacity || request.ProjectionExpression || request.ExpressionAttributeNames;
  if(hasKwargs) {
    tube.write(tube.cbor.encodeMapStreamHeader());
    if(request.ConsistentRead !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ConsistentRead));
  tube.write(tube.cbor.encodeBoolean(request.ConsistentRead));
}

    if(request.ReturnConsumedCapacity !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnConsumedCapacity));
  tube.write(tube.cbor.encodeInt(Constants.ReturnConsumedCapacityValues[request.ReturnConsumedCapacity.toUpperCase()]));
}

    // This operation has expressions, so deal with those together
    let exprBytes = encoder.encodeExpressions(request).otherBytes;
    if(exprBytes !== null) {
      tube.write(exprBytes);
    }

    tube.write(tube.cbor.encodeStreamBreak());
  } else {
    tube.write(tube.cbor.encodeNull());
  }

  tube.flush();
};

exports.write_putItem_N2106490455_1 = function(request, tube) {
  let encoder = new Encoders(tube.cbor, request._keySchema, request._attrNames, request._attrListId);

  tube.write(tube.cbor.encodeInt(1));
  tube.write(tube.cbor.encodeInt(-2106490455));
  tube.write(encoder.encodeTableName(request.TableName));

  tube.write(encoder.encodeKey(request.Item));

  tube.write(encoder.encodeValues(request.Item));

  let hasKwargs = request.ReturnValues || request.ReturnConsumedCapacity || request.ReturnItemCollectionMetrics || request.ConditionExpression || request.ExpressionAttributeNames || request.ExpressionAttributeValues;
  if(hasKwargs) {
    tube.write(tube.cbor.encodeMapStreamHeader());
    if(request.ReturnValues !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnValues));
  tube.write(tube.cbor.encodeInt(Constants.ReturnValuesValues[request.ReturnValues.toUpperCase()]));
}

    if(request.ReturnConsumedCapacity !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnConsumedCapacity));
  tube.write(tube.cbor.encodeInt(Constants.ReturnConsumedCapacityValues[request.ReturnConsumedCapacity.toUpperCase()]));
}

    if(request.ReturnItemCollectionMetrics !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnItemCollectionMetrics));
  tube.write(tube.cbor.encodeInt(Constants.ReturnItemCollectionMetricsValues[request.ReturnItemCollectionMetrics.toUpperCase()]));
}

    if(request.ConditionExpression !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ConditionExpression));
  tube.write(tube.cbor.encodeBinary(request.ConditionExpression));
}

    // This operation has expressions, so deal with those together
    let exprBytes = encoder.encodeExpressions(request).otherBytes;
    if(exprBytes !== null) {
      tube.write(exprBytes);
    }

    tube.write(tube.cbor.encodeStreamBreak());
  } else {
    tube.write(tube.cbor.encodeNull());
  }

  tube.flush();
};

exports.write_query_N931250863_1 = function(request, tube) {
  let encoder = new Encoders(tube.cbor, request._keySchema);

  tube.write(tube.cbor.encodeInt(1));
  tube.write(tube.cbor.encodeInt(-931250863));
  tube.write(encoder.encodeTableName(request.TableName));

  let exprResult = encoder.encodeExpressions(request);
  if(exprResult.keyCondBytes !== null) {
    tube.write(exprResult.keyCondBytes);
  } else {
    tube.write(tube.cbor.encodeNull());
  }

  let hasKwargs = request.IndexName || request.Select || request.Limit || request.ConsistentRead || request.ScanIndexForward || request.ExclusiveStartKey || request.ReturnConsumedCapacity || request.ProjectionExpression || request.FilterExpression || request.ExpressionAttributeNames || request.ExpressionAttributeValues;
  if(hasKwargs) {
    tube.write(tube.cbor.encodeMapStreamHeader());
    if(request.IndexName !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.IndexName));
  tube.write(tube.cbor.encodeBinary(request.IndexName));
}

    if(request.Select !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.Select));
  tube.write(tube.cbor.encodeInt(Constants.SelectValues[request.Select.toUpperCase()]));
}

    if(request.Limit !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.Limit));
  tube.write(tube.cbor.encodeInt(request.Limit));
}

    if(request.ConsistentRead !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ConsistentRead));
  tube.write(tube.cbor.encodeInt(request.ConsistentRead ? 1 : 0));
}

    if(request.ScanIndexForward !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ScanIndexForward));
  tube.write(tube.cbor.encodeInt(request.ScanIndexForward ? 1 : 0));
}

    if(request.ExclusiveStartKey !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ExclusiveStartKey));
  // No encoder for map so use custom encoder
  tube.write(encoder.encode_ExclusiveStartKey(request));
}

    if(request.ReturnConsumedCapacity !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnConsumedCapacity));
  tube.write(tube.cbor.encodeInt(Constants.ReturnConsumedCapacityValues[request.ReturnConsumedCapacity.toUpperCase()]));
}

    // This operation has expressions, so deal with those together
    // For Query, the expressions are already eval'd for KeyCondExpr
    let exprBytes = exprResult.otherBytes;
    if(exprBytes !== null) {
      tube.write(exprBytes);
    }

    tube.write(tube.cbor.encodeStreamBreak());
  } else {
    tube.write(tube.cbor.encodeNull());
  }

  tube.flush();
};

exports.write_scan_N1875390620_1 = function(request, tube) {
  let encoder = new Encoders(tube.cbor, request._keySchema);

  tube.write(tube.cbor.encodeInt(1));
  tube.write(tube.cbor.encodeInt(-1875390620));
  tube.write(encoder.encodeTableName(request.TableName));

  let hasKwargs = request.IndexName || request.Limit || request.Select || request.ExclusiveStartKey || request.ReturnConsumedCapacity || request.TotalSegments || request.Segment || request.ProjectionExpression || request.FilterExpression || request.ExpressionAttributeNames || request.ExpressionAttributeValues || request.ConsistentRead;
  if(hasKwargs) {
    tube.write(tube.cbor.encodeMapStreamHeader());
    if(request.IndexName !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.IndexName));
  tube.write(tube.cbor.encodeBinary(request.IndexName));
}

    if(request.Limit !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.Limit));
  tube.write(tube.cbor.encodeInt(request.Limit));
}

    if(request.Select !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.Select));
  tube.write(tube.cbor.encodeInt(Constants.SelectValues[request.Select.toUpperCase()]));
}

    if(request.ExclusiveStartKey !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ExclusiveStartKey));
  // No encoder for map so use custom encoder
  tube.write(encoder.encode_ExclusiveStartKey(request));
}

    if(request.ReturnConsumedCapacity !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnConsumedCapacity));
  tube.write(tube.cbor.encodeInt(Constants.ReturnConsumedCapacityValues[request.ReturnConsumedCapacity.toUpperCase()]));
}

    if(request.TotalSegments !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.TotalSegments));
  tube.write(tube.cbor.encodeInt(request.TotalSegments));
}

    if(request.Segment !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.Segment));
  tube.write(tube.cbor.encodeInt(request.Segment));
}

    if(request.ConsistentRead !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ConsistentRead));
  tube.write(tube.cbor.encodeInt(request.ConsistentRead ? 1 : 0));
}

    // This operation has expressions, so deal with those together
    let exprBytes = encoder.encodeExpressions(request).otherBytes;
    if(exprBytes !== null) {
      tube.write(exprBytes);
    }

    tube.write(tube.cbor.encodeStreamBreak());
  } else {
    tube.write(tube.cbor.encodeNull());
  }

  tube.flush();
};

exports.write_updateItem_1425579023_1 = function(request, tube) {
  let encoder = new Encoders(tube.cbor, request._keySchema);

  tube.write(tube.cbor.encodeInt(1));
  tube.write(tube.cbor.encodeInt(1425579023));
  tube.write(encoder.encodeTableName(request.TableName));

  tube.write(encoder.encodeKey(request.Key));

  let hasKwargs = request.ReturnValues || request.ReturnConsumedCapacity || request.ReturnItemCollectionMetrics || request.UpdateExpression || request.ConditionExpression || request.ExpressionAttributeNames || request.ExpressionAttributeValues;
  if(hasKwargs) {
    tube.write(tube.cbor.encodeMapStreamHeader());
    if(request.ReturnValues !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnValues));
  tube.write(tube.cbor.encodeInt(Constants.ReturnValuesValues[request.ReturnValues.toUpperCase()]));
}

    if(request.ReturnConsumedCapacity !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnConsumedCapacity));
  tube.write(tube.cbor.encodeInt(Constants.ReturnConsumedCapacityValues[request.ReturnConsumedCapacity.toUpperCase()]));
}

    if(request.ReturnItemCollectionMetrics !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ReturnItemCollectionMetrics));
  tube.write(tube.cbor.encodeInt(Constants.ReturnItemCollectionMetricsValues[request.ReturnItemCollectionMetrics.toUpperCase()]));
}

    if(request.ConditionExpression !== undefined) {
  tube.write(tube.cbor.encodeInt(Constants.DaxDataRequestParam.ConditionExpression));
  tube.write(tube.cbor.encodeBinary(request.ConditionExpression));
}

    // This operation has expressions, so deal with those together
    let exprBytes = encoder.encodeExpressions(request).otherBytes;
    if(exprBytes !== null) {
      tube.write(exprBytes);
    }

    tube.write(tube.cbor.encodeStreamBreak());
  } else {
    tube.write(tube.cbor.encodeNull());
  }

  tube.flush();
};

